import Flutter
import UIKit
import ESPProvision

public class SwiftEspBluetoothProvisioningPlugin: NSObject, FlutterPlugin, ESPDeviceConnectionDelegate {
    
    var permissionManager = PermissionManager()
    var channel: FlutterMethodChannel?
    var selectedDevice: ESPDevice?
    var pop: String = ""
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "esp_ble_provisioning", binaryMessenger: registrar.messenger())
        let instance = SwiftEspBluetoothProvisioningPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let method = call.method
        let arguments: Dictionary<String, Any>? = call.arguments as! Dictionary<String, Any>?
        
        print(method, arguments ?? "no arguments")
        
        if (method == "getPlatformVersion") {
            result(UIDevice.current.systemVersion)
        }
        else if (method == "scanBleDevice") {
            print("scan ble devices")
            ESPProvisionManager.shared.searchESPDevices(devicePrefix:"", transport:.ble, security:.secure) { deviceList, _ in
                print("scan response", deviceList?.count ?? 0)
                var devices: [[String: String]] = [[String: String]]()
                deviceList?.forEach({ dev in
                    print(dev.name)
                    let json = (["name": dev.name, "uuid": dev.name])
                    devices.append(json)
                })
                self.channel?.invokeMethod(method, arguments: devices)
            }
            result(true)
        }
        else if (method == "connectBleDevice") {
            let name = arguments?["name"] as! String
            pop = arguments?["pin"] as! String
            print("scan ble devices to connect")
            ESPProvisionManager.shared.searchESPDevices(devicePrefix:"", transport:.ble, security:.secure) { deviceList, _ in
                print("scan response", deviceList?.count ?? 0)
                deviceList?.forEach({ dev in
                    print(dev.name)
                    if (dev.name == name)
                    {
                        print("found device")
                        self.selectedDevice = dev
                    }
                })
                
                if (self.selectedDevice != nil)
                {
                    self.selectedDevice!.connect(delegate: self) { status in
                        print(status)
                        var respStatus = "failure"
                        switch status {
                        case .connected:
                            respStatus = "success"
                            break
                            
                        default:
                            respStatus = "failure"
                            break
                        }
                        self.channel?.invokeMethod(method, arguments: respStatus)
                    }
                }
            }
            result(true)
        }
        else if (method == "provisionDevice") {
            let ssid = arguments?["ssid"] as! String
            let password = arguments?["password"] as! String
            print("start provisioning " + ssid + password)
            if (self.selectedDevice != nil)
            {
                self.selectedDevice!.provision(ssid: ssid, passPhrase: password) { status in
                    print(status)
                    var respStatus = "DEVICE_PROVISIONING_FAILED"
                    switch status {
                    case .success:
                        respStatus = "DEVICE_PROVISIONING_SUCCESS"
                        break

                    case .configApplied:
                        respStatus = "DEVICE_PROVISIONING_SUCCESS" //WIFI_CONFIG_APPLIED??
                        break

                    default:
                        respStatus = "DEVICE_PROVISIONING_FAILED"
                        break
                    }
                    //self.channel?.invokeMethod(method, arguments: respStatus)
                    result(respStatus)
                }
            }
            else
            {
                result("DEVICE_PROVISIONING_FAILED")
            }
            //result(true)
        }
        else if (method == "requestPermission") {
            permissionManager.requestPermission()
            result("requested")
        }
        else if (method == "permissionState") {
            result(permissionManager.permissionStatus)
        }
        else if (method == "bluetoothState") {
            result(permissionManager.state)
        }
        else if (method.contains("bluetoothStateListener")) {
            print("bluetooth state listener")
            permissionManager.bluetoothState(method: method) { (state) in
                print(state)
                self.channel?.invokeMethod(method, arguments: state)
            }
            result(true)
        }
        else {
            result (false)
        }
    }
    
    public func getProofOfPossesion(forDevice: ESPDevice, completionHandler: @escaping (String) -> Void) {
        print("get POP")
        completionHandler(pop);
    }

}
extension String {
    func removingWhitespaces() -> String {
        return components(separatedBy: .whitespaces).joined()
    }
}
