import 'dart:async';

import 'package:flutter/services.dart';

import 'models/BleDevice.dart';
import 'models/enums.dart';

class EspBleProvisioning {
  static const MethodChannel _channel = const MethodChannel('esp_ble_provisioning');
  Map<String, StreamController<BluetoothState>> _stateStreamPool = {};
  Map<String, StreamController<String>> _connectionStatus = {};
  Map<String, StreamController<String>> _provisionStatus = {};
  Map<String, StreamController<List<BleDevices>>> _ideviceStreamPool = {};
  List<BleDevices> device = [];
  int _stateScannerCount = 0;
  int _provisionStatusCount = 0;

  EspBleProvisioning() {
    _channel.setMethodCallHandler((call) {
      String method = call.method;
      print(method);
      if (method.contains("bluetoothStateListener")) {
        print("State Stream");
        print(call.arguments);
        _stateStreamPool[call.method]!.add(_getBluetoothStateFromKey(call.arguments));
      } else if (method.contains("scanBleDevice") && _ideviceStreamPool.containsKey(call.method)) {
        print("Device Response");
        print(call.arguments);
        List<BleDevices> beacons = call.arguments.map<BleDevices>((b) => BleDevices.fromMap(b))
            .toList();
        device.clear();
        device.addAll(beacons);
        print("Device : ${beacons.length}");
        _ideviceStreamPool[call.method]!.add(beacons);
      } else if (method.contains("connectBleDevice")) {
        print("Connection Response : ${call.arguments}");
        _connectionStatus[call.method]!.add((call.arguments));
      }
      // else if (method.contains("provisionDevice")) {
      //   print("Provision Response : ${call.arguments}");
      //   // HashMap<String, String> map = call.arguments;
      //   // print("Provision Response : $map");
      //
      //   _provisionStatus[call.method].add((call.arguments));
      // }
      return printFunct();
    });
  }

  /// dummy function return future<dynamic>s
   Future printFunct() async {return;}

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  /// Asks for location permission from the platfrom
  void requestPermission() {
    print("************request permission");
    _channel.invokeMethod<String>("requestPermission");
  }

  /// Stream which splits the bluetooth adapter state
  Stream<BluetoothState> getBluetoothStateStream() {
    String method = "bluetoothStateListener#${++_stateScannerCount}";
    _stateStreamPool[method] = _getNewBluetoothStateStreamController<BluetoothState>(method);
    _channel.invokeMethod(method);
    return _stateStreamPool[method]!.stream;
  }

  /// Scans and sends the scan response on the foreground ie Dart side
  Stream<List<BleDevices>> scanBluetoothDevice() {
    print("scanning starts");
    String method = "scanBleDevice";
      _ideviceStreamPool[method] =
          _getNewIBeaconStreamController<List<BleDevices>>(method);
      _channel.invokeMethod(method);

    return _ideviceStreamPool[method]!.stream.map((beacons) {
      List<BleDevices> freshIBeacons = beacons.toList();
      print("Stream --------------------->");
      return freshIBeacons;
    });
  }

  /// Stream which splits the bluetooth adapter state
  Stream<String> connectToBluetoothDevice(String name, String pin) {
    String method = "connectBleDevice";
    _connectionStatus[method] = _getConnectionStreamController<String>(method);
    _channel.invokeMethod(method, {"name": name, "pin": pin});
    print(_connectionStatus[method]!.stream.toString());
    return _connectionStatus[method]!.stream;
  }

  Stream<String> provisionDevice(String ssid, String password) {
    String method = "provisionDevice#${++_provisionStatusCount}";
    _provisionStatus[method] = _getProvisionStreamController<String>(method);
    _channel.invokeMethod(method, {"ssid": ssid, "password": password});
    print(_provisionStatus[method]!.stream.toString());
    return _provisionStatus[method]!.stream;
  }

  Future<ProvisionState> startProvisioning(String ssid, String password) async {
    print("************Trying to provision device");
    var status = await _channel.invokeMethod('provisionDevice', {"ssid": ssid, "password": password});
    print("Before conversion------------------> $status");
    print("Status------------------> ${_getStatusFromKey(status)}" +
        DateTime.now().toString());
    return _getStatusFromKey(status);
  }

  StreamController<T> _getNewBluetoothStateStreamController<T>(
      String identifier) {
    print("_getNewBluetoothStateStreamController");
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelBluetoothStateListener", {
        "methodName": identifier,
      });
    };
    print(controller);
    return controller;
  }

  StreamController<T> _getNewIBeaconStreamController<T>(String identifier) {
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelStream", {
        "methodName": identifier,
      });
      _ideviceStreamPool.remove(identifier);
    };
    print(controller);
    return controller;
  }

  StreamController<T> _getConnectionStreamController<T>(String identifier) {
    print("_getConnectionStreamController");
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelBluetoothStateListener", {
        "methodName": identifier,
      });
    };
    print(controller);
    return controller;
  }

  StreamController<T> _getProvisionStreamController<T>(String identifier) {
    print("_getProvisionStreamController");
    StreamController<T> controller = StreamController<T>();
    controller.onCancel = () {
      _channel.invokeMethod("cancelBluetoothStateListener", {
        "methodName": identifier,
      });
    };
    print(controller);
    return controller;
  }

  /// Retrives the hardware state of the bluetooth adapter on the device.
  Future<BluetoothState> getBluetoothState() async {
    return _getBluetoothStateFromKey(
        await _channel.invokeMethod("bluetoothState"));
  }
}

BluetoothState _getBluetoothStateFromKey(String key) {
  switch (key) {
    case "on":
      return BluetoothState.ON;
    case "off":
      return BluetoothState.OFF;
    default:
      return BluetoothState.UNKNOWN;
  }
}

ProvisionState _getStatusFromKey(String key) {
  print(key);
  switch (key) {
    case "CREATE_SESSION_FAILED":
      return ProvisionState.CREATE_SESSION_FAILED;
    case "WIFI_CONFIG_SENT":
      return ProvisionState.WIFI_CONFIG_SENT;
    case "WIFI_CONFIG_FAILED":
      return ProvisionState.WIFI_CONFIG_FAILED;
    case "WIFI_CONFIG_APPLIED":
      return ProvisionState.WIFI_CONFIG_APPLIED;
    case "DEVICE_PROVISIONING_SUCCESS":
      return ProvisionState.DEVICE_PROVISIONING_SUCCESS;
    default:
      return ProvisionState.DEVICE_PROVISIONING_FAILED;
  }
}
